@@ -13,7 +13,7 @@ extern "C" {
 
 
 /** Defines the Baresip version string */
-#define BARESIP_VERSION "0.5.8"
+#define BARESIP_VERSION "0.5.10"
 
 
 #ifndef NET_MAX_NS
@@ -21,6 +21,12 @@ extern "C" {
 #endif
 
 
+/*
+ * Clock-rate for video timestamp
+ */
+#define VIDEO_TIMEBASE 1000000U
+
+
 /* forward declarations */
 struct sa;
 struct sdp_media;
@@ -48,8 +54,14 @@ struct account;
 
 int account_alloc(struct account **accp, const char *sipaddr);
 int account_debug(struct re_printf *pf, const struct account *acc);
+int account_set_auth_user(struct account *acc, const char *user);
 int account_set_auth_pass(struct account *acc, const char *pass);
+int account_set_outbound(struct account *acc, const char *ob, unsigned ix);
+int account_set_sipnat(struct account *acc, const char *sipnat);
 int account_set_display_name(struct account *acc, const char *dname);
+int account_set_regint(struct account *acc, uint32_t regint);
+int account_set_mediaenc(struct account *acc, const char *mediaenc);
+int account_set_audio_codecs(struct account *acc, const char *codecs);
 int account_auth(const struct account *acc, char **username, char **password,
 		 const char *realm);
 struct list *account_aucodecl(const struct account *acc);
@@ -59,13 +71,16 @@ uint32_t account_regint(const struct account *acc);
 uint32_t account_pubint(const struct account *acc);
 uint32_t account_ptime(const struct account *acc);
 enum answermode account_answermode(const struct account *acc);
+const char *account_display_name(const struct account *acc);
 const char *account_aor(const struct account *acc);
 const char *account_auth_user(const struct account *acc);
 const char *account_auth_pass(const struct account *acc);
 const char *account_outbound(const struct account *acc, unsigned ix);
+const char *account_sipnat(const struct account *acc);
 const char *account_stun_user(const struct account *acc);
 const char *account_stun_pass(const struct account *acc);
 const char *account_stun_host(const struct account *acc);
+const char *account_mediaenc(const struct account *acc);
 
 
 /*
@@ -92,6 +107,13 @@ enum call_event {
 	CALL_EVENT_CLOSED,
 	CALL_EVENT_TRANSFER,
 	CALL_EVENT_TRANSFER_FAILED,
+	CALL_EVENT_MENC,
+};
+
+enum menc_event {
+	MENC_EVENT_SECURE,
+	MENC_EVENT_VERIFY_REQUEST,
+	MENC_EVENT_PEER_VERIFIED,
 };
 
 struct call;
@@ -182,7 +204,7 @@ struct config_sip {
 
 /** Call config */
 struct config_call {
-	uint32_t local_timeout; /**< Incoming call timeout [sec] 0=off */
+	uint32_t local_timeout; /**< Incoming call timeout [sec] 0=off    */
 	uint32_t max_calls;     /**< Maximum number of calls, 0=unlimited */
 };
 
@@ -219,7 +241,7 @@ struct config_video {
 	char disp_dev[128];     /**< Video display device           */
 	unsigned width, height; /**< Video resolution               */
 	uint32_t bitrate;       /**< Encoder bitrate in [bit/s]     */
-	uint32_t fps;           /**< Video framerate                */
+	double fps;             /**< Video framerate                */
 	bool fullscreen;        /**< Enable fullscreen display      */
 	int enc_fmt;            /**< Encoder pixelfmt (enum vidfmt) */
 };
@@ -239,7 +261,7 @@ struct config_avt {
 
 /* Network */
 struct config_net {
-	char ifname[16];        /**< Bind to interface (optional)   */
+	char ifname[64];        /**< Bind to interface (optional)   */
 	struct {
 		char addr[64];
 	} nsv[NET_MAX_NS];      /**< Configured DNS nameservers     */
@@ -367,8 +389,8 @@ struct ausrc_st;
 
 /** Audio Source parameters */
 struct ausrc_prm {
-	uint32_t   srate;       /**< Sampling rate in [Hz] */
-	uint8_t    ch;          /**< Number of channels    */
+	uint32_t   srate;       /**< Sampling rate in [Hz]      */
+	uint8_t    ch;          /**< Number of channels         */
 	uint32_t   ptime;       /**< Wanted packet-time in [ms] */
 	int        fmt;         /**< Sample format (enum aufmt) */
 };
@@ -400,8 +422,8 @@ struct auplay_st;
 
 /** Audio Player parameters */
 struct auplay_prm {
-	uint32_t   srate;       /**< Sampling rate in [Hz] */
-	uint8_t    ch;          /**< Number of channels    */
+	uint32_t   srate;       /**< Sampling rate in [Hz]      */
+	uint8_t    ch;          /**< Number of channels         */
 	uint32_t   ptime;       /**< Wanted packet-time in [ms] */
 	int        fmt;         /**< Sample format (enum aufmt) */
 };
@@ -446,12 +468,14 @@ struct aufilt_prm {
 };
 
 typedef int (aufilt_encupd_h)(struct aufilt_enc_st **stp, void **ctx,
-			      const struct aufilt *af, struct aufilt_prm *prm);
+			      const struct aufilt *af, struct aufilt_prm *prm,
+			      const struct audio *au);
 typedef int (aufilt_encode_h)(struct aufilt_enc_st *st,
 			      int16_t *sampv, size_t *sampc);
 
 typedef int (aufilt_decupd_h)(struct aufilt_dec_st **stp, void **ctx,
-			      const struct aufilt *af, struct aufilt_prm *prm);
+			      const struct aufilt *af, struct aufilt_prm *prm,
+			      const struct audio *au);
 typedef int (aufilt_decode_h)(struct aufilt_dec_st *st,
 			      int16_t *sampv, size_t *sampc);
 
@@ -510,8 +534,11 @@ struct menc_media;
 
 typedef void (menc_error_h)(int err, void *arg);
 
+typedef void (menc_event_h)(enum menc_event event, const char *prm, void *arg);
+
 typedef int  (menc_sess_h)(struct menc_sess **sessp, struct sdp_session *sdp,
-			   bool offerer, menc_error_h *errorh, void *arg);
+			   bool offerer, menc_event_h *eventh,
+			   menc_error_h *errorh, void *arg);
 
 typedef int  (menc_media_h)(struct menc_media **mp, struct menc_sess *sess,
 			    struct rtp_sock *rtp, int proto,
@@ -592,6 +619,9 @@ enum ua_event {
 	UA_EVENT_CALL_DTMF_START,
 	UA_EVENT_CALL_DTMF_END,
 	UA_EVENT_CALL_RTCP,
+	UA_EVENT_CALL_MENC,
+	UA_EVENT_VU_TX,
+	UA_EVENT_VU_RX,
 
 	UA_EVENT_MAX,
 };
@@ -625,6 +655,7 @@ int  ua_debug(struct re_printf *pf, const struct ua *ua);
 int  ua_print_calls(struct re_printf *pf, const struct ua *ua);
 int  ua_print_status(struct re_printf *pf, const struct ua *ua);
 int  ua_print_supported(struct re_printf *pf, const struct ua *ua);
+int  ua_update_account(struct ua *ua);
 int  ua_register(struct ua *ua);
 void ua_unregister(struct ua *ua);
 bool ua_isregistered(const struct ua *ua);
@@ -640,6 +671,9 @@ struct list    *ua_calls(const struct ua *ua);
 enum presence_status ua_presence_status(const struct ua *ua);
 void ua_presence_status_set(struct ua *ua, const enum presence_status status);
 void ua_set_media_af(struct ua *ua, int af_media);
+void ua_set_catchall(struct ua *ua, bool enabled);
+void ua_event(struct ua *ua, enum ua_event ev, struct call *call,
+	      const char *fmt, ...);
 
 
 /* One instance */
@@ -671,8 +705,8 @@ struct sipevent_sock *uag_sipevent_sock(void);
  */
 
 struct ui_sub {
-	struct list uil;  /**< List of UIs (struct ui) */
-	struct cmd_ctx *uictx;
+	struct list uil;        /**< List of UIs (struct ui) */
+	struct cmd_ctx *uictx;  /**< Command context         */
 };
 
 typedef int  (ui_output_h)(const char *str);
@@ -691,6 +725,7 @@ void ui_reset(struct ui_sub *uis);
 void ui_input_key(struct ui_sub *uis, char key, struct re_printf *pf);
 void ui_input_str(const char *str);
 int  ui_input_pl(struct re_printf *pf, const struct pl *pl);
+int  ui_input_long_command(struct re_printf *pf, const struct pl *pl);
 void ui_output(struct ui_sub *uis, const char *fmt, ...);
 bool ui_isediting(const struct ui_sub *uis);
 int  ui_password_prompt(char **passwordp);
@@ -701,9 +736,9 @@ int  ui_password_prompt(char **passwordp);
  */
 
 /* special keys */
-#define KEYCODE_NONE   (0x00)
+#define KEYCODE_NONE   (0x00)    /* No key           */
 #define KEYCODE_REL    (0x04)    /* Key was released */
-#define KEYCODE_ESC    (0x1b)
+#define KEYCODE_ESC    (0x1b)    /* Escape key       */
 
 
 /** Command flags */
@@ -760,10 +795,18 @@ struct vidsrc_st;
 /** Video Source parameters */
 struct vidsrc_prm {
 	int orient;       /**< Wanted picture orientation (enum vidorient) */
-	int fps;          /**< Wanted framerate                            */
+	double fps;       /**< Wanted framerate                            */
 };
 
-typedef void (vidsrc_frame_h)(struct vidframe *frame, void *arg);
+/**
+ * Provides video frames to the core
+ *
+ * @param frame     Video frame
+ * @param timestamp Frame timestamp in VIDEO_TIMEBASE units
+ * @param arg       Handler argument
+ */
+typedef void (vidsrc_frame_h)(struct vidframe *frame, uint64_t timestamp,
+			      void *arg);
 typedef void (vidsrc_error_h)(int err, void *arg);
 
 typedef int  (vidsrc_alloc_h)(struct vidsrc_st **vsp, const struct vidsrc *vs,
@@ -776,6 +819,14 @@ typedef int  (vidsrc_alloc_h)(struct vidsrc_st **vsp, const struct vidsrc *vs,
 typedef void (vidsrc_update_h)(struct vidsrc_st *st, struct vidsrc_prm *prm,
 			       const char *dev);
 
+/** Defines a video source */
+struct vidsrc {
+	struct le         le;
+	const char       *name;
+	vidsrc_alloc_h   *alloch;
+	vidsrc_update_h  *updateh;
+};
+
 int vidsrc_register(struct vidsrc **vp, struct list *vidsrcl, const char *name,
 		    vidsrc_alloc_h *alloch, vidsrc_update_h *updateh);
 const struct vidsrc *vidsrc_find(const struct list *vidsrcl, const char *name);
@@ -784,6 +835,7 @@ int vidsrc_alloc(struct vidsrc_st **stp, struct list *vidsrcl,
 		 struct media_ctx **ctx, struct vidsrc_prm *prm,
 		 const struct vidsz *size, const char *fmt, const char *dev,
 		 vidsrc_frame_h *frameh, vidsrc_error_h *errorh, void *arg);
+struct vidsrc *vidsrc_get(struct vidsrc_st *st);
 
 
 /*
@@ -811,6 +863,16 @@ typedef int  (vidisp_disp_h)(struct vidisp_st *st, const char *title,
 			     const struct vidframe *frame);
 typedef void (vidisp_hide_h)(struct vidisp_st *st);
 
+/** Defines a Video display */
+struct vidisp {
+	struct le        le;
+	const char      *name;
+	vidisp_alloc_h  *alloch;
+	vidisp_update_h *updateh;
+	vidisp_disp_h   *disph;
+	vidisp_hide_h   *hideh;
+};
+
 int vidisp_register(struct vidisp **vp, struct list *vidispl, const char *name,
 		    vidisp_alloc_h *alloch, vidisp_update_h *updateh,
 		    vidisp_disp_h *disph, vidisp_hide_h *hideh);
@@ -821,6 +883,7 @@ int vidisp_alloc(struct vidisp_st **stp, struct list *vidispl,
 int vidisp_display(struct vidisp_st *st, const char *title,
 		   const struct vidframe *frame);
 const struct vidisp *vidisp_find(const struct list *vidispl, const char *name);
+struct vidisp *vidisp_get(struct vidisp_st *st);
 
 
 /*
@@ -829,8 +892,8 @@ const struct vidisp *vidisp_find(const struct list *vidispl, const char *name);
 
 /** Audio Codec parameters */
 struct auenc_param {
-	uint32_t ptime;  /**< Packet time in [ms]   */
-	uint32_t bitrate;/**< Wanted bitrate in [bit/s] */
+	uint32_t ptime;    /**< Packet time in [ms]       */
+	uint32_t bitrate;  /**< Wanted bitrate in [bit/s] */
 };
 
 struct auenc_state;
@@ -884,7 +947,7 @@ const struct aucodec *aucodec_find(const struct list *aucodecl,
 struct videnc_param {
 	unsigned bitrate;  /**< Encoder bitrate in [bit/s] */
 	unsigned pktsize;  /**< RTP packetsize in [bytes]  */
-	unsigned fps;      /**< Video framerate            */
+	double fps;        /**< Video framerate (max)      */
 	uint32_t max_fs;
 };
 
@@ -892,7 +955,7 @@ struct videnc_state;
 struct viddec_state;
 struct vidcodec;
 
-typedef int (videnc_packet_h)(bool marker, uint32_t rtp_ts,
+typedef int (videnc_packet_h)(bool marker, uint64_t rtp_ts,
 			      const uint8_t *hdr, size_t hdr_len,
 			      const uint8_t *pld, size_t pld_len,
 			      void *arg);
@@ -901,8 +964,10 @@ typedef int (videnc_update_h)(struct videnc_state **vesp,
 			      const struct vidcodec *vc,
 			      struct videnc_param *prm, const char *fmtp,
 			      videnc_packet_h *pkth, void *arg);
+
 typedef int (videnc_encode_h)(struct videnc_state *ves, bool update,
-			      const struct vidframe *frame);
+			      const struct vidframe *frame,
+			      uint64_t timestamp);
 
 typedef int (viddec_update_h)(struct viddec_state **vdsp,
 			      const struct vidcodec *vc, const char *fmtp);
@@ -992,8 +1057,11 @@ int  audio_set_player(struct audio *au, const char *mod, const char *device);
 void audio_encoder_cycle(struct audio *audio);
 int  audio_level_get(const struct audio *au, double *level);
 int  audio_debug(struct re_printf *pf, const struct audio *a);
-struct stream *audio_strm(const struct audio *a);
-int audio_set_bitrate(struct audio *au, uint32_t bitrate);
+struct stream *audio_strm(const struct audio *au);
+int  audio_set_bitrate(struct audio *au, uint32_t bitrate);
+bool audio_rxaubuf_started(const struct audio *au);
+void audio_stop(struct audio *a);
+void audio_set_hold(struct audio *au, bool hold);
 
 
 /*
@@ -1011,9 +1079,11 @@ int   video_set_source(struct video *v, const char *name, const char *dev);
 void  video_set_devicename(struct video *v, const char *src, const char *disp);
 void  video_encoder_cycle(struct video *video);
 int   video_debug(struct re_printf *pf, const struct video *v);
-uint32_t video_calc_rtp_timestamp(int64_t pts, unsigned fps);
-double video_calc_seconds(uint32_t rtp_ts);
+uint64_t video_calc_rtp_timestamp(int64_t pts, double fps);
+uint64_t video_calc_rtp_timestamp_fix(uint64_t timestamp);
+double video_calc_seconds(uint64_t rtp_ts);
 struct stream *video_strm(const struct video *v);
+double video_timestamp_to_seconds(uint64_t timestamp);
 
 
 /*
@@ -1021,6 +1091,7 @@ struct stream *video_strm(const struct video *v);
  */
 
 const struct rtcp_stats *stream_rtcp_stats(const struct stream *strm);
+struct call *stream_call(const struct stream *strm);
 
 
 /*
@@ -1143,12 +1214,13 @@ int h264_fu_hdr_decode(struct h264_fu *fu, struct mbuf *mb);
 
 const uint8_t *h264_find_startcode(const uint8_t *p, const uint8_t *end);
 
-int h264_packetize(uint32_t rtp_ts, const uint8_t *buf, size_t len,
+int h264_packetize(uint64_t rtp_ts, const uint8_t *buf, size_t len,
 		   size_t pktsize, videnc_packet_h *pkth, void *arg);
 int h264_nal_send(bool first, bool last,
-		  bool marker, uint32_t ihdr, uint32_t rtp_ts,
+		  bool marker, uint32_t ihdr, uint64_t rtp_ts,
 		  const uint8_t *buf, size_t size, size_t maxsz,
 		  videnc_packet_h *pkth, void *arg);
+const char *h264_nalunit_name(int type);
 static inline bool h264_is_keyframe(int type)
 {
 	return type == H264_NAL_SPS;
@@ -1187,6 +1259,13 @@ int event_encode_dict(struct odict *od, struct ua *ua, enum ua_event ev,
 		      struct call *call, const char *prm);
 
 
+/*
+ * Timer
+ */
+
+uint64_t tmr_jiffies_usec(void);
+
+
 /*
  * Baresip instance
  */
