@@ -20,12 +20,6 @@
 #include "avcodec.h"
 
 
-#if LIBAVUTIL_VERSION_MAJOR < 52
-#define AV_PIX_FMT_YUV420P PIX_FMT_YUV420P
-#define AV_PIX_FMT_NV12    PIX_FMT_NV12
-#endif
-
-
 #ifndef AV_INPUT_BUFFER_MIN_SIZE
 #define AV_INPUT_BUFFER_MIN_SIZE FF_MIN_BUFFER_SIZE
 #endif
@@ -48,7 +42,6 @@ struct videnc_state {
 	AVFrame *pict;
 	struct mbuf *mb;
 	size_t sz_max; /* todo: figure out proper buffer size */
-	int64_t pts;
 	struct mbuf *mb_frag;
 	struct videnc_param encprm;
 	struct vidsz encsize;
@@ -219,6 +212,9 @@ static int open_encoder(struct videnc_state *st,
 
 	/* params to avoid libavcodec/x264 default preset error */
 	if (st->codec_id == AV_CODEC_ID_H264) {
+
+		av_opt_set(st->ctx->priv_data, "profile", "baseline", 0);
+
 		st->ctx->me_range = 16;
 		st->ctx->qmin = 10;
 		st->ctx->qmax = 51;
@@ -346,7 +342,7 @@ static void param_handler(const struct pl *name, const struct pl *val,
 }
 
 
-static int general_packetize(uint32_t rtp_ts, struct mbuf *mb, size_t pktsize,
+static int general_packetize(uint64_t rtp_ts, struct mbuf *mb, size_t pktsize,
 			     videnc_packet_h *pkth, void *arg)
 {
 	int err = 0;
@@ -371,7 +367,7 @@ static int general_packetize(uint32_t rtp_ts, struct mbuf *mb, size_t pktsize,
 
 
 static int h263_packetize(struct videnc_state *st,
-			  uint32_t rtp_ts, struct mbuf *mb,
+			  uint64_t rtp_ts, struct mbuf *mb,
 			  videnc_packet_h *pkth, void *arg)
 {
 	struct h263_strm h263_strm;
@@ -448,32 +444,22 @@ static int open_encoder_x264(struct videnc_state *st, struct videnc_param *prm,
 	xprm.analyse.b_transform_8x8 = 0;
 	xprm.analyse.i_me_method = X264_ME_DIA;
 	xprm.analyse.i_subpel_refine = 0;
-#if X264_BUILD >= 59
 	xprm.rc.i_aq_mode = 0;
-#endif
 	xprm.analyse.b_mixed_references = 0;
 	xprm.analyse.i_trellis = 0;
-#if X264_BUILD >= 63
 	xprm.i_bframe_adaptive = X264_B_ADAPT_NONE;
-#endif
-#if X264_BUILD >= 70
 	xprm.rc.b_mb_tree = 0;
-#endif
 
 	/* slice-based threading (--tune=zerolatency) */
-#if X264_BUILD >= 80
 	xprm.rc.i_lookahead = 0;
 	xprm.i_sync_lookahead = 0;
 	xprm.i_bframe = 0;
-#endif
 
 	/* put SPS/PPS before each keyframe */
 	xprm.b_repeat_headers = 1;
 
-#if X264_BUILD >= 82
 	/* needed for x264_encoder_intra_refresh() */
 	xprm.b_intra_refresh = 1;
-#endif
 
 	if (st->x264)
 		x264_encoder_close(st->x264);
@@ -547,7 +533,7 @@ int encode_update(struct videnc_state **vesp, const struct vidcodec *vc,
 		fmt_param_apply(&sdp_fmtp, param_handler, st);
 	}
 
-	debug("avcodec: video encoder %s: %d fps, %d bit/s, pktsize=%u\n",
+	debug("avcodec: video encoder %s: %.2f fps, %d bit/s, pktsize=%u\n",
 	      vc->name, prm->fps, prm->bitrate, prm->pktsize);
 
  out:
@@ -562,14 +548,15 @@ int encode_update(struct videnc_state **vesp, const struct vidcodec *vc,
 
 #ifdef USE_X264
 int encode_x264(struct videnc_state *st, bool update,
-		const struct vidframe *frame)
+		const struct vidframe *frame, uint64_t timestamp)
 {
 	x264_picture_t pic_in, pic_out;
 	x264_nal_t *nal;
 	int i_nal;
 	int i, err, ret;
 	int csp, pln;
-	uint32_t ts;
+	int64_t input_pts;
+	uint64_t ts;
 
 	if (!st || !frame)
 		return EINVAL;
@@ -605,17 +592,23 @@ int encode_x264(struct videnc_state *st, bool update,
 	}
 
 	if (update) {
-#if X264_BUILD >= 95
 		x264_encoder_intra_refresh(st->x264);
-#endif
 		debug("avcodec: x264 picture update\n");
 	}
 
 	x264_picture_init(&pic_in);
 
+	/*
+	 * We use the video source timestamp as PTS.
+	 * Since the PTS is in time_base units (derived from FPS) and
+	 * the input and output has the same units, this should work
+	 * fine, as long as the real FPS is less than the MAX FPS.
+	 */
+	input_pts = timestamp;
+
 	pic_in.i_type = update ? X264_TYPE_IDR : X264_TYPE_AUTO;
 	pic_in.i_qpplus1 = 0;
-	pic_in.i_pts = ++st->pts;
+	pic_in.i_pts = input_pts;
 
 	pic_in.img.i_csp = csp;
 	pic_in.img.i_plane = pln;
@@ -631,7 +624,7 @@ int encode_x264(struct videnc_state *st, bool update,
 	if (i_nal == 0)
 		return 0;
 
-	ts = video_calc_rtp_timestamp(pic_out.i_pts, st->encprm.fps);
+	ts = video_calc_rtp_timestamp_fix(pic_out.i_pts);
 
 	err = 0;
 	for (i=0; i<i_nal && !err; i++) {
@@ -666,11 +659,13 @@ int encode_x264(struct videnc_state *st, bool update,
 #endif
 
 
-int encode(struct videnc_state *st, bool update, const struct vidframe *frame)
+int encode(struct videnc_state *st, bool update, const struct vidframe *frame,
+	   uint64_t timestamp)
 {
 	int i, err, ret;
 	int pix_fmt;
-	uint32_t ts;
+	int64_t pts;
+	uint64_t ts;
 
 	if (!st || !frame)
 		return EINVAL;
@@ -708,7 +703,7 @@ int encode(struct videnc_state *st, bool update, const struct vidframe *frame)
 		st->pict->data[i]     = frame->data[i];
 		st->pict->linesize[i] = frame->linesize[i];
 	}
-	st->pict->pts = st->pts++;
+	st->pict->pts = timestamp;
 	if (update) {
 		debug("avcodec: encoder picture update\n");
 		st->pict->key_frame = 1;
@@ -743,7 +738,7 @@ int encode(struct videnc_state *st, bool update, const struct vidframe *frame)
 			return 0;
 		}
 
-		ts = video_calc_rtp_timestamp(pkt->dts, st->encprm.fps);
+		pts = pkt->dts;
 
 		err = mbuf_write_mem(st->mb, pkt->data, pkt->size);
 		st->mb->pos = 0;
@@ -772,7 +767,7 @@ int encode(struct videnc_state *st, bool update, const struct vidframe *frame)
 
 		mbuf_set_end(st->mb, avpkt.size);
 
-		ts = video_calc_rtp_timestamp(avpkt.dts, st->encprm.fps);
+		pts = avpkt.dts;
 
 	} while (0);
 #else
@@ -790,9 +785,11 @@ int encode(struct videnc_state *st, bool update, const struct vidframe *frame)
 
 	mbuf_set_end(st->mb, ret);
 
-	ts = video_calc_rtp_timestamp(st->pict->pts, st->encprm.fps);
+	pts = st->pict->pts;
 #endif
 
+	ts = video_calc_rtp_timestamp_fix(pts);
+
 	switch (st->codec_id) {
 
 	case AV_CODEC_ID_H263:
