@@ -66,7 +66,6 @@ struct vidsrc_st {
 	vidsrc_frame_h *frameh;
 	void *arg;
 	int sindex;
-	int fps;
 };
 
 
@@ -101,6 +100,9 @@ static void handle_packet(struct vidsrc_st *st, AVPacket *pkt)
 	struct vidframe vf;
 	struct vidsz sz;
 	unsigned i;
+	int64_t pts;
+	uint64_t timestamp;
+	const AVRational time_base = st->time_base;
 
 	if (st->codec) {
 		int got_pict, ret;
@@ -148,6 +150,11 @@ static void handle_packet(struct vidsrc_st *st, AVPacket *pkt)
 		return;
 	}
 
+	pts = frame->pts;
+
+	/* convert timestamp */
+	timestamp = pts * VIDEO_TIMEBASE * time_base.num / time_base.den;
+
 #if LIBAVCODEC_VERSION_INT >= ((53<<16)+(5<<8)+0)
 	switch (frame->format) {
 
@@ -173,7 +180,7 @@ static void handle_packet(struct vidsrc_st *st, AVPacket *pkt)
 		vf.linesize[i] = frame->linesize[i];
 	}
 
-	st->frameh(&vf, st->arg);
+	st->frameh(&vf, timestamp, st->arg);
 
  out:
 	if (frame) {
@@ -244,7 +251,7 @@ static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 	bool found_stream = false;
 	uint32_t i;
 	int ret, err = 0;
-	int input_fps = 0;
+	double input_fps = 0;
 
 	(void)mctx;
 	(void)errorh;
@@ -262,7 +269,6 @@ static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 	st->sz     = *size;
 	st->frameh = frameh;
 	st->arg    = arg;
-	st->fps    = prm->fps;
 
 	/*
 	 * avformat_open_input() was added in lavf 53.2.0 according to
@@ -283,7 +289,7 @@ static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 	/* Params */
 	memset(&prms, 0, sizeof(prms));
 
-	prms.time_base          = (AVRational){1, prm->fps};
+	prms.time_base          = av_d2q(prm->fps, INT_MAX);
 	prms.channels           = 1;
 	prms.width              = size->w;
 	prms.height             = size->h;
@@ -320,7 +326,6 @@ static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 	for (i=0; i<st->ic->nb_streams; i++) {
 		const struct AVStream *strm = st->ic->streams[i];
 		AVCodecContext *ctx;
-		double dfps;
 
 #if LIBAVFORMAT_VERSION_INT >= ((57<<16) + (33<<8) + 100)
 
@@ -355,14 +360,17 @@ static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 		st->sindex = strm->index;
 		st->time_base = strm->time_base;
 
-		dfps = av_q2d(strm->avg_frame_rate);
-		input_fps = (int)dfps;
-		if (st->fps != input_fps) {
-			info("avformat: updating %i fps from config to native "
-				"input material fps %i\n", st->fps, input_fps);
-			st->fps = input_fps;
+		input_fps = av_q2d(strm->avg_frame_rate);
+		if (prm->fps != input_fps) {
+			info("avformat: updating %.2f fps from config"
+			     " to native "
+			     "input material fps %.2f\n",
+			     prm->fps, input_fps);
+
+			prm->fps = input_fps;
+
 #if LIBAVFORMAT_VERSION_INT < ((52<<16) + (110<<8) + 0)
-			prms.time_base = (AVRational){1, st->fps};
+			prms.time_base = av_d2q(input_fps, INT_MAX);
 #endif
 		}
 
@@ -414,14 +422,18 @@ static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 static int module_init(void)
 {
 	/* register all codecs, demux and protocols */
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 9, 100)
 	avcodec_register_all();
+#endif
 	avdevice_register_all();
 
 #if LIBAVFORMAT_VERSION_INT >= ((53<<16) + (13<<8) + 0)
 	avformat_network_init();
 #endif
 
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 9, 100)
 	av_register_all();
+#endif
 
 	return vidsrc_register(&mod_avf, baresip_vidsrcl(),
 			       "avformat", alloc, NULL);
