@@ -23,6 +23,7 @@
 struct vumeter_enc {
 	struct aufilt_enc_st af;  /* inheritance */
 	struct tmr tmr;
+	const struct audio *au;
 	double avg_rec;
 	volatile bool started;
 };
@@ -30,11 +31,31 @@ struct vumeter_enc {
 struct vumeter_dec {
 	struct aufilt_dec_st af;  /* inheritance */
 	struct tmr tmr;
+	const struct audio *au;
 	double avg_play;
 	volatile bool started;
 };
 
 
+static void send_event(const struct audio *au, enum ua_event ev, double value)
+{
+	struct stream *strm;
+	struct call *call;
+	struct ua *ua;
+
+	/* get the stream from the audio object */
+	strm = audio_strm(au);
+
+	/* get the call from the stream object */
+	call = stream_call(strm);
+
+	/* get the useragent from the call object */
+	ua = call_get_ua(call);
+
+	ua_event(ua, ev, call, "%.2f", value);
+}
+
+
 static void enc_destructor(void *arg)
 {
 	struct vumeter_enc *st = arg;
@@ -86,10 +107,13 @@ static void enc_tmr_handler(void *arg)
 {
 	struct vumeter_enc *st = arg;
 
-	tmr_start(&st->tmr, 100, enc_tmr_handler, st);
+	tmr_start(&st->tmr, 500, enc_tmr_handler, st);
 
-	if (st->started)
+	if (st->started) {
 		print_vumeter(60, 31, st->avg_rec);
+
+		send_event(st->au, UA_EVENT_VU_TX, st->avg_rec);
+	}
 }
 
 
@@ -97,15 +121,19 @@ static void dec_tmr_handler(void *arg)
 {
 	struct vumeter_dec *st = arg;
 
-	tmr_start(&st->tmr, 100, dec_tmr_handler, st);
+	tmr_start(&st->tmr, 500, dec_tmr_handler, st);
 
-	if (st->started)
+	if (st->started) {
 		print_vumeter(80, 32, st->avg_play);
+
+		send_event(st->au, UA_EVENT_VU_RX, st->avg_play);
+	}
 }
 
 
 static int encode_update(struct aufilt_enc_st **stp, void **ctx,
-			 const struct aufilt *af, struct aufilt_prm *prm)
+			 const struct aufilt *af, struct aufilt_prm *prm,
+			 const struct audio *au)
 {
 	struct vumeter_enc *st;
 	(void)ctx;
@@ -121,6 +149,7 @@ static int encode_update(struct aufilt_enc_st **stp, void **ctx,
 	if (!st)
 		return ENOMEM;
 
+	st->au = au;
 	tmr_start(&st->tmr, 100, enc_tmr_handler, st);
 
 	*stp = (struct aufilt_enc_st *)st;
@@ -130,7 +159,8 @@ static int encode_update(struct aufilt_enc_st **stp, void **ctx,
 
 
 static int decode_update(struct aufilt_dec_st **stp, void **ctx,
-			 const struct aufilt *af, struct aufilt_prm *prm)
+			 const struct aufilt *af, struct aufilt_prm *prm,
+			 const struct audio *au)
 {
 	struct vumeter_dec *st;
 	(void)ctx;
@@ -146,6 +176,7 @@ static int decode_update(struct aufilt_dec_st **stp, void **ctx,
 	if (!st)
 		return ENOMEM;
 
+	st->au = au;
 	tmr_start(&st->tmr, 100, dec_tmr_handler, st);
 
 	*stp = (struct aufilt_dec_st *)st;
