@@ -97,7 +97,7 @@ struct autx {
 	bool muted;                   /**< Audio source is muted           */
 	int cur_key;                  /**< Currently transmitted event     */
 	enum aufmt src_fmt;           /**< Sample format for audio source  */
-	enum aufmt enc_fmt;
+	enum aufmt enc_fmt;           /**< Sample format for encoder       */
 	bool need_conv;               /**< Sample format conversion needed */
 
 	struct {
@@ -146,17 +146,17 @@ struct aurx {
 	int16_t *sampv_rs;            /**< Sample buffer for resampler     */
 	uint32_t ptime;               /**< Packet time for receiving       */
 	int pt;                       /**< Payload type for incoming RTP   */
-	double level_last;
-	bool level_set;
+	double level_last;            /**< Last audio level value [dBov]   */
+	bool level_set;               /**< True if level_last is set       */
 	enum aufmt play_fmt;          /**< Sample format for audio playback*/
-	enum aufmt dec_fmt;
+	enum aufmt dec_fmt;           /**< Sample format for decoder       */
 	bool need_conv;               /**< Sample format conversion needed */
 	struct timestamp_recv ts_recv;/**< Receive timestamp state         */
-	uint64_t n_discard;
 
 	struct {
 		uint64_t aubuf_overrun;
 		uint64_t aubuf_underrun;
+		uint64_t n_discard;
 	} stats;
 };
 
@@ -171,6 +171,7 @@ struct audio {
 	struct config_audio cfg;      /**< Audio configuration             */
 	bool started;                 /**< Stream is started flag          */
 	bool level_enabled;           /**< Audio level RTP ext. enabled    */
+	bool hold;                    /**< Local hold flag                 */
 	unsigned extmap_aulevel;      /**< ID Range 1-14 inclusive         */
 	audio_event_h *eventh;        /**< Event handler                   */
 	audio_err_h *errh;            /**< Audio error handler             */
@@ -310,7 +311,7 @@ static inline double calc_ptime(size_t nsamp, uint32_t srate, uint8_t channels)
 }
 
 
-/**
+/*
  * Get the DSP samplerate for an audio-codec
  */
 static inline uint32_t get_srate(const struct aucodec *ac)
@@ -322,7 +323,7 @@ static inline uint32_t get_srate(const struct aucodec *ac)
 }
 
 
-/**
+/*
  * Get the DSP channels for an audio-codec (exception for MPA)
  */
 static inline uint32_t get_ch(const struct aucodec *ac)
@@ -635,9 +636,9 @@ static void check_telev(struct audio *a, struct autx *tx)
  *
  * @note The sample format is set in rx->play_fmt
  *
- * @param buf Buffer to fill with audio samples
- * @param sz  Number of bytes in buffer
- * @param arg Handler argument
+ * @param sampv  Buffer to fill with audio samples
+ * @param sampc  Number of samples in buffer
+ * @param arg    Handler argument
  */
 static void auplay_write_handler(void *sampv, size_t sampc, void *arg)
 {
@@ -648,8 +649,10 @@ static void auplay_write_handler(void *sampv, size_t sampc, void *arg)
 
 		++rx->stats.aubuf_underrun;
 
+#if 0
 		debug("audio: rx aubuf underrun (total %llu)\n",
 		      rx->stats.aubuf_underrun);
+#endif
 	}
 
 	aubuf_read(rx->aubuf, sampv, num_bytes);
@@ -663,9 +666,9 @@ static void auplay_write_handler(void *sampv, size_t sampc, void *arg)
  *
  * @note This function may be called from any thread
  *
- * @param buf Buffer with audio samples
- * @param sz  Number of bytes in buffer
- * @param arg Handler argument
+ * @param sampv  Buffer with audio samples
+ * @param sampc  Number of samples in buffer
+ * @param arg    Handler argument
  */
 static void ausrc_read_handler(const void *sampv, size_t sampc, void *arg)
 {
@@ -994,7 +997,7 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 #endif
 
 	if (discard) {
-		++a->rx.n_discard;
+		++rx->stats.n_discard;
 		return;
 	}
 
@@ -1088,7 +1091,6 @@ int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
 	struct aurx *rx;
 	struct le *le;
 	int err;
-	(void)offerer;
 
 	if (!ap || !cfg)
 		return EINVAL;
@@ -1342,7 +1344,7 @@ static int aufilt_setup(struct audio *a)
 		void *ctx = NULL;
 
 		if (af->encupdh) {
-			err |= af->encupdh(&encst, &ctx, af, &encprm);
+			err |= af->encupdh(&encst, &ctx, af, &encprm, a);
 			if (err)
 				break;
 
@@ -1351,7 +1353,7 @@ static int aufilt_setup(struct audio *a)
 		}
 
 		if (af->decupdh) {
-			err |= af->decupdh(&decst, &ctx, af, &decprm);
+			err |= af->decupdh(&decst, &ctx, af, &decprm, a);
 			if (err)
 				break;
 
@@ -1501,7 +1503,7 @@ static int start_source(struct autx *tx, struct audio *a)
 	}
 
 	/* Start Audio Source */
-	if (!tx->ausrc && ausrc_find(baresip_ausrcl(), NULL)) {
+	if (!tx->ausrc && ausrc_find(baresip_ausrcl(), NULL) && !a->hold) {
 
 		struct ausrc_prm prm;
 		size_t sz;
@@ -1535,6 +1537,10 @@ static int start_source(struct autx *tx, struct audio *a)
 		}
 
 		switch (a->cfg.txmode) {
+
+		case AUDIO_MODE_POLL:
+			break;
+
 #ifdef HAVE_PTHREAD
 		case AUDIO_MODE_THREAD:
 			if (!tx->u.thr.run) {
@@ -1550,7 +1556,9 @@ static int start_source(struct autx *tx, struct audio *a)
 #endif
 
 		default:
-			break;
+			warning("audio: tx mode not supported (%d)\n",
+				a->cfg.txmode);
+			return ENOTSUP;
 		}
 
 		tx->ausrc_prm = prm;
@@ -1577,6 +1585,8 @@ int audio_start(struct audio *a)
 	if (!a)
 		return EINVAL;
 
+	debug("audio: start\n");
+
 	/* Audio filter */
 	if (!list_isempty(baresip_aufiltl())) {
 		err = aufilt_setup(a);
@@ -1749,9 +1759,16 @@ void audio_encoder_cycle(struct audio *audio)
 }
 
 
-struct stream *audio_strm(const struct audio *a)
+/**
+ * Get the RTP Stream object from an Audio object
+ *
+ * @param au  Audio object
+ *
+ * @return RTP Stream object
+ */
+struct stream *audio_strm(const struct audio *au)
 {
-	return a ? a->strm : NULL;
+	return au ? au->strm : NULL;
 }
 
 
@@ -1962,7 +1979,9 @@ int audio_debug(struct re_printf *pf, const struct audio *a)
 				     tx->ausrc_prm.ch),
 			  tx->stats.aubuf_overrun,
 			  tx->stats.aubuf_underrun);
-	err |= re_hprintf(pf, "       source: %s\n",
+	err |= re_hprintf(pf, "       source: %s,%s %s\n",
+			  tx->ausrc ? tx->ausrc->as->name : "none",
+			  tx->device,
 			  aufmt_name(tx->src_fmt));
 	err |= re_hprintf(pf, "       time = %.3f sec\n",
 			  autx_calc_seconds(tx));
@@ -1984,9 +2003,12 @@ int audio_debug(struct re_printf *pf, const struct audio *a)
 			  rx->stats.aubuf_overrun,
 			  rx->stats.aubuf_underrun
 			  );
-	err |= re_hprintf(pf, "       player: %s\n", aufmt_name(rx->play_fmt));
+	err |= re_hprintf(pf, "       player: %s,%s %s\n",
+			  rx->auplay ? rx->auplay->ap->name : "none",
+			  rx->device,
+			  aufmt_name(rx->play_fmt));
 	err |= re_hprintf(pf, "       n_discard:%llu\n",
-			  rx->n_discard);
+			  rx->stats.n_discard);
 	if (rx->level_set) {
 		err |= re_hprintf(pf, "       level %.3f dBov\n",
 				  rx->level_last);
@@ -2011,6 +2033,14 @@ int audio_debug(struct re_printf *pf, const struct audio *a)
 }
 
 
+/**
+ * Set the audio source and player device name. This function does not
+ * change the state of the audio source/player.
+ *
+ * @param a     Audio object
+ * @param src   Audio source device name
+ * @param play  Audio player device name
+ */
 void audio_set_devicename(struct audio *a, const char *src, const char *play)
 {
 	if (!a)
@@ -2021,6 +2051,16 @@ void audio_set_devicename(struct audio *a, const char *src, const char *play)
 }
 
 
+/**
+ * Set the audio source state to a new audio source module and device.
+ * The current audio source will be stopped.
+ *
+ * @param au     Audio object
+ * @param mod    Audio source module (NULL to stop)
+ * @param device Audio source device name
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int audio_set_source(struct audio *au, const char *mod, const char *device)
 {
 	struct autx *tx;
@@ -2034,19 +2074,32 @@ int audio_set_source(struct audio *au, const char *mod, const char *device)
 	/* stop the audio device first */
 	tx->ausrc = mem_deref(tx->ausrc);
 
-	err = ausrc_alloc(&tx->ausrc, baresip_ausrcl(),
-			  NULL, mod, &tx->ausrc_prm, device,
-			  ausrc_read_handler, ausrc_error_handler, au);
-	if (err) {
-		warning("audio: set_source failed (%s.%s): %m\n",
-			mod, device, err);
-		return err;
+	if (str_isset(mod)) {
+
+		err = ausrc_alloc(&tx->ausrc, baresip_ausrcl(),
+				  NULL, mod, &tx->ausrc_prm, device,
+				  ausrc_read_handler, ausrc_error_handler, au);
+		if (err) {
+			warning("audio: set_source failed (%s.%s): %m\n",
+				mod, device, err);
+			return err;
+		}
 	}
 
 	return 0;
 }
 
 
+/**
+ * Set the audio player state to a new audio player module and device.
+ * The current audio player will be stopped.
+ *
+ * @param au     Audio object
+ * @param mod    Audio player module (NULL to stop)
+ * @param device Audio player device name
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int audio_set_player(struct audio *au, const char *mod, const char *device)
 {
 	struct aurx *rx;
@@ -2060,13 +2113,16 @@ int audio_set_player(struct audio *au, const char *mod, const char *device)
 	/* stop the audio device first */
 	rx->auplay = mem_deref(rx->auplay);
 
-	err = auplay_alloc(&rx->auplay, baresip_auplayl(),
-			   mod, &rx->auplay_prm, device,
-			   auplay_write_handler, rx);
-	if (err) {
-		warning("audio: set_player failed (%s.%s): %m\n",
-			mod, device, err);
-		return err;
+	if (str_isset(mod)) {
+
+		err = auplay_alloc(&rx->auplay, baresip_auplayl(),
+				   mod, &rx->auplay_prm, device,
+				   auplay_write_handler, rx);
+		if (err) {
+			warning("audio: set_player failed (%s.%s): %m\n",
+				mod, device, err);
+			return err;
+		}
 	}
 
 	return 0;
@@ -2139,6 +2195,14 @@ int audio_print_rtpstat(struct re_printf *pf, const struct audio *a)
 }
 
 
+/**
+ * Set the bitrate for the audio encoder
+ *
+ * @param au      Audio object
+ * @param bitrate Encoder bitrate in [bits/s]
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int audio_set_bitrate(struct audio *au, uint32_t bitrate)
 {
 	struct autx *tx;
@@ -2178,3 +2242,38 @@ int audio_set_bitrate(struct audio *au, uint32_t bitrate)
 
 	return 0;
 }
+
+
+/**
+ * Check if audio receiving has started
+ *
+ * @param au      Audio object
+ *
+ * @return True if started, otherwise false
+ */
+bool audio_rxaubuf_started(const struct audio *au)
+{
+	const struct aurx *rx;
+
+	if (!au)
+		return false;
+
+	rx = &au->rx;
+
+	return rx->aubuf_started;
+}
+
+
+/**
+ * Set the audio stream on hold
+ *
+ * @param au    Audio object
+ * @param hold  True to hold, false to resume
+ */
+void audio_set_hold(struct audio *au, bool hold)
+{
+	if (!au)
+		return;
+
+	au->hold = hold;
+}
