@@ -67,7 +67,7 @@ struct call {
 	time_t time_stop;         /**< Time when call stopped               */
 	bool outgoing;            /**< True if outgoing, false if incoming  */
 	bool got_offer;           /**< Got SDP Offer from Peer              */
-	bool on_hold;             /**< True if call is on hold              */
+	bool on_hold;             /**< True if call is on hold (local)      */
 	struct mnat_sess *mnats;  /**< Media NAT session                    */
 	bool mnat_wait;           /**< Waiting for MNAT to establish        */
 	struct menc_sess *mencs;  /**< Media encryption session state       */
@@ -115,6 +115,8 @@ static void call_stream_start(struct call *call, bool active)
 	const struct sdp_format *sc;
 	int err;
 
+	debug("call: stream start (active=%d)\n", active);
+
 	/* Audio Stream */
 	sc = sdp_media_rformat(stream_sdpmedia(audio_strm(call->audio)), NULL);
 	if (sc) {
@@ -240,7 +242,7 @@ static void invite_timeout(void *arg)
 }
 
 
-/** Called when all media streams are established */
+/* Called when all media streams are established */
 static void mnat_handler(int err, uint16_t scode, const char *reason,
 			 void *arg)
 {
@@ -450,6 +452,20 @@ static void menc_error_handler(int err, void *arg)
 }
 
 
+static void menc_event_handler(const enum menc_event event,
+			       const char *prm, void *arg)
+{
+	struct call *call = arg;
+	MAGIC_CHECK(call);
+
+	if (strlen(prm) > 0)
+		call_event_handler(call, CALL_EVENT_MENC, "%u,%s", event,
+				   prm);
+	else
+		call_event_handler(call, CALL_EVENT_MENC, "%u", event);
+}
+
+
 static void stream_error_handler(struct stream *strm, int err, void *arg)
 {
 	struct call *call = arg;
@@ -582,8 +598,9 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 	if (acc->menc) {
 		if (acc->menc->sessh) {
 			err = acc->menc->sessh(&call->mencs, call->sdp,
-						!got_offer,
-						menc_error_handler, call);
+					       !got_offer,
+					       menc_event_handler,
+					       menc_error_handler, call);
 			if (err) {
 				warning("call: mediaenc session: %m\n", err);
 				goto out;
@@ -721,6 +738,8 @@ int call_modify(struct call *call)
 	if (!call)
 		return EINVAL;
 
+	debug("call: modify\n");
+
 	err = call_sdp_get(call, &desc, true);
 	if (!err)
 		err = sipsess_modify(call->sess, desc);
@@ -1002,6 +1021,9 @@ int call_status(struct re_printf *pf, const struct call *call)
 		err |= video_print(pf, call->video);
 #endif
 
+	/* remove old junk */
+	err |= re_hprintf(pf, "    ");
+
 	return err;
 }
 
@@ -1109,6 +1131,8 @@ static int sipsess_answer_handler(const struct sip_msg *msg, void *arg)
 
 	MAGIC_CHECK(call);
 
+	debug("call: got SDP answer (%zu bytes)\n", mbuf_get_left(msg->mb));
+
 	if (msg_ctype_cmp(&msg->ctyp, "multipart", "mixed"))
 		(void)sdp_decode_multipart(&msg->ctyp.params, msg->mb);
 
@@ -1168,14 +1192,14 @@ static void call_handle_info_req(struct call *call, const struct sip_msg *req)
 	bool pfu;
 	int err;
 
-	(void)call;
-
 	pl_set_mbuf(&body, req->mb);
 
 	err = mctrl_handle_media_control(&body, &pfu);
 	if (err)
 		return;
 
+	debug("call: receive media control: fast_update=%d\n", pfu);
+
 	if (pfu) {
 		video_update_picture(call->video);
 	}
@@ -1838,6 +1862,13 @@ void call_set_xrtpstat(struct call *call)
 }
 
 
+/**
+ * Check if a call is locally on hold
+ *
+ * @param call Call object
+ *
+ * @return True if on hold (local), otherwise false
+ */
 bool call_is_onhold(const struct call *call)
 {
 	return call ? call->on_hold : false;
