@@ -183,6 +183,7 @@ int reg_add(struct list *lst, struct ua *ua, int regid)
 int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 		 uint32_t regint, const char *outbound)
 {
+	struct account *acc;
 	const char *routev[1];
 	int err;
 
@@ -191,10 +192,12 @@ int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 
 	reg->scode = 0;
 	routev[0] = outbound;
+	acc = ua_account(reg->ua);
 
 	reg->sipreg = mem_deref(reg->sipreg);
 	err = sipreg_register(&reg->sipreg, uag_sip(), reg_uri,
-			      ua_aor(reg->ua), ua_aor(reg->ua),
+			      ua_aor(reg->ua),
+			      acc ? acc->dispname : NULL, ua_aor(reg->ua),
 			      regint, ua_local_cuser(reg->ua),
 			      routev[0] ? routev : NULL,
 			      routev[0] ? 1 : 0,
