@@ -338,6 +338,8 @@ static int stream_sock_alloc(struct stream *s, int af)
 
 	udp_rxsz_set(rtp_sock(s->rtp), RTP_RECV_SIZE);
 
+	udp_sockbuf_set(rtp_sock(s->rtp), 65536);
+
 	return 0;
 }
 
@@ -501,7 +503,9 @@ int stream_send(struct stream *s, bool ext, bool marker, int pt, uint32_t ts,
 
 	if (!sa_isset(sdp_media_raddr(s->sdp), SA_ALL))
 		return 0;
-	if (sdp_media_dir(s->sdp) != SDP_SENDRECV)
+	if (!(sdp_media_rdir(s->sdp) & SDP_SENDONLY))
+		return 0;
+	if (s->hold)
 		return 0;
 
 	metric_add_packet(&s->metric_tx, mbuf_get_left(mb));
@@ -614,6 +618,7 @@ void stream_hold(struct stream *s, bool hold)
 	if (!s)
 		return;
 
+	s->hold = hold;
 	sdp_media_set_ldir(s->sdp, hold ? SDP_SENDONLY : SDP_SENDRECV);
 }
 
@@ -738,3 +743,16 @@ const struct rtcp_stats *stream_rtcp_stats(const struct stream *strm)
 {
 	return strm ? &strm->rtcp_stats : NULL;
 }
+
+
+/**
+ * Get the call object from the stream
+ *
+ * @param strm Stream object
+ *
+ * @return Call object
+ */
+struct call *stream_call(const struct stream *strm)
+{
+	return strm ? strm->call : NULL;
+}
