@@ -18,7 +18,6 @@
 /** Defines a SIP User Agent object */
 struct ua {
 	MAGIC_DECL                   /**< Magic number for struct ua         */
-	struct ua **uap;             /**< Pointer to application's ua        */
 	struct le le;                /**< Linked list element                */
 	struct account *acc;         /**< Account Parameters                 */
 	struct list regl;            /**< List of Register clients           */
@@ -30,6 +29,7 @@ struct ua {
 	int af;                      /**< Preferred Address Family           */
 	int af_media;                /**< Preferred Address Family for media */
 	enum presence_status my_status; /**< Presence Status                 */
+	bool catchall;               /**< Catch all inbound requests         */
 };
 
 struct ua_eh {
@@ -201,6 +201,8 @@ int ua_register(struct ua *ua)
 				   acc->regint, acc->outboundv[i]);
 		if (err) {
 			warning("ua: SIP register failed: %m\n", err);
+
+			ua_event(ua, UA_EVENT_REGISTER_FAIL, NULL, "%m", err);
 			goto out;
 		}
 	}
@@ -385,6 +387,10 @@ static void call_event_handler(struct call *call, enum call_event ev,
 	case CALL_EVENT_TRANSFER_FAILED:
 		ua_event(ua, UA_EVENT_CALL_TRANSFER_FAILED, call, str);
 		break;
+
+	case CALL_EVENT_MENC:
+		ua_event(ua, UA_EVENT_CALL_MENC, call, str);
+		break;
 	}
 }
 
@@ -523,11 +529,6 @@ static void ua_destructor(void *arg)
 {
 	struct ua *ua = arg;
 
-	if (ua->uap) {
-		*ua->uap = NULL;
-		ua->uap = NULL;
-	}
-
 	list_unlink(&ua->le);
 
 	if (!list_isempty(&ua->regl))
@@ -571,7 +572,8 @@ static void add_extension(struct ua *ua, const char *extension)
 	struct pl e;
 
 	if (ua->extensionc >= ARRAY_SIZE(ua->extensionv)) {
-		warning("ua: maximum %u number of SIP extensions\n");
+		warning("ua: maximum %zu number of SIP extensions\n",
+			ARRAY_SIZE(ua->extensionv));
 		return;
 	}
 
@@ -581,6 +583,46 @@ static void add_extension(struct ua *ua, const char *extension)
 }
 
 
+static int create_register_clients(struct ua *ua)
+{
+	int err = 0;
+
+	/* Register clients */
+	if (uag.cfg && str_isset(uag.cfg->uuid))
+	        add_extension(ua, "gruu");
+
+	if (0 == str_casecmp(ua->acc->sipnat, "outbound")) {
+
+		size_t i;
+
+		add_extension(ua, "path");
+		add_extension(ua, "outbound");
+
+		if (!str_isset(uag.cfg->uuid)) {
+
+			warning("ua: outbound requires valid UUID!\n");
+			err = ENOSYS;
+			goto out;
+		}
+
+		for (i=0; i<ARRAY_SIZE(ua->acc->outboundv); i++) {
+
+			if (ua->acc->outboundv[i] && ua->acc->regint) {
+				err = reg_add(&ua->regl, ua, (int)i+1);
+				if (err)
+					break;
+			}
+		}
+	}
+	else if (ua->acc->regint) {
+		err = reg_add(&ua->regl, ua, 0);
+	}
+
+ out:
+	return err;
+}
+
+
 /**
  * Allocate a SIP User-Agent
  *
@@ -647,36 +689,7 @@ int ua_alloc(struct ua **uap, const char *aor)
 			  ua->acc->menc->id);
 	}
 
-	/* Register clients */
-	if (uag.cfg && str_isset(uag.cfg->uuid))
-	        add_extension(ua, "gruu");
-
-	if (0 == str_casecmp(ua->acc->sipnat, "outbound")) {
-
-		size_t i;
-
-		add_extension(ua, "path");
-		add_extension(ua, "outbound");
-
-		if (!str_isset(uag.cfg->uuid)) {
-
-			warning("ua: outbound requires valid UUID!\n");
-			err = ENOSYS;
-			goto out;
-		}
-
-		for (i=0; i<ARRAY_SIZE(ua->acc->outboundv); i++) {
-
-			if (ua->acc->outboundv[i] && ua->acc->regint) {
-				err = reg_add(&ua->regl, ua, (int)i+1);
-				if (err)
-					break;
-			}
-		}
-	}
-	else if (ua->acc->regint) {
-		err = reg_add(&ua->regl, ua, 0);
-	}
+	err = create_register_clients(ua);
 	if (err)
 		goto out;
 
@@ -693,20 +706,32 @@ int ua_alloc(struct ua **uap, const char *aor)
 	mem_deref(buf);
 	if (err)
 		mem_deref(ua);
-	else if (uap) {
+	else if (uap)
 		*uap = ua;
 
-		ua->uap = uap;
-	}
-
 	return err;
 }
 
 
+int ua_update_account(struct ua *ua)
+{
+	if (!ua)
+		return EINVAL;
+
+	/* clear extensions and reg clients */
+	ua->extensionc = 0;
+	list_flush(&ua->regl);
+
+	return create_register_clients(ua);
+}
+
+
 static int uri_complete(struct ua *ua, struct mbuf *buf, const char *uri)
 {
 	size_t len;
 	int err = 0;
+	bool uri_is_ip;
+	struct sa sa_addr;
 
 	/* Skip initial whitespace */
 	while (isspace(*uri))
@@ -720,8 +745,13 @@ static int uri_complete(struct ua *ua, struct mbuf *buf, const char *uri)
 
 	err |= mbuf_write_str(buf, uri);
 
-	/* Append domain if missing */
-	if (0 != re_regex(uri, len, "[^@]+@[^]+", NULL, NULL)) {
+	/* Append domain if missing and uri is not IP address */
+
+	/* check if uri is valid IP address */
+	uri_is_ip = (0 == sa_set_str(&sa_addr, uri, 0));
+
+	if (0 != re_regex(uri, len, "[^@]+@[^]+", NULL, NULL) &&
+		1 != uri_is_ip) {
 #if HAVE_INET6
 		if (AF_INET6 == ua->acc->luri.af)
 			err |= mbuf_printf(buf, "@[%r]",
@@ -1233,6 +1263,10 @@ static void sipsess_conn_handler(const struct sip_msg *msg, void *arg)
 
 	(void)arg;
 
+	debug("ua: sipsess connect via %s %J --> %J\n",
+	      sip_transp_name(msg->tp),
+	      &msg->src, &msg->dst);
+
 	ua = uag_find(&msg->uri.user);
 	if (!ua) {
 		warning("ua: %r: UA not found: %r\n",
@@ -1659,6 +1693,14 @@ struct ua *uag_find(const struct pl *cuser)
 			return ua;
 	}
 
+	/* Last resort, try any catchall UAs */
+	for (le = uag.ual.head; le; le = le->next) {
+		struct ua *ua = le->data;
+
+		if (ua->catchall)
+			return ua;
+	}
+
 	return NULL;
 }
 
@@ -1896,6 +1938,22 @@ void ua_set_media_af(struct ua *ua, int af_media)
 }
 
 
+/**
+ * Enable handling of all inbound requests, even if
+ * the request uri is not matching.
+ *
+ * @param ua      User-Agent
+ * @param enabled True to enable, false to disable
+ */
+void ua_set_catchall(struct ua *ua, bool enabled)
+{
+	if (!ua)
+		return;
+
+	ua->catchall = enabled;
+}
+
+
 int uag_set_extra_params(const char *eprm)
 {
 	uag.eprm = mem_deref(uag.eprm);
