@@ -82,7 +82,7 @@ struct vtx {
 	struct vidsrc_prm vsrc_prm;        /**< Video source parameters   */
 	struct vidsz vsrc_size;            /**< Video source size         */
 	struct vidsrc_st *vsrc;            /**< Video source              */
-	struct lock *lock;                 /**< Lock for encoder          */
+	struct lock *lock_enc;             /**< Lock for encoder          */
 	struct vidframe *frame;            /**< Source frame              */
 	struct vidframe *mute_frame;       /**< Frame with muted video    */
 	struct lock *lock_tx;              /**< Protect the sendq         */
@@ -96,9 +96,14 @@ struct vtx {
 	bool picup;                        /**< Send picture update       */
 	bool muted;                        /**< Muted flag                */
 	int frames;                        /**< Number of frames sent     */
-	int efps;                          /**< Estimated frame-rate      */
-	uint32_t ts_min;
-	uint32_t ts_max;
+	double efps;                       /**< Estimated frame-rate      */
+	uint64_t ts_base;                  /**< First RTP timestamp sent  */
+	uint64_t ts_last;                  /**< Last RTP timestamp sent   */
+
+	/** Statistics */
+	struct {
+		uint64_t src_frames;       /**< Total frames from vidsrc  */
+	} stats;
 };
 
 
@@ -133,11 +138,16 @@ struct vrx {
 	char device[128];                  /**< Display device name       */
 	int pt_rx;                         /**< Incoming RTP payload type */
 	int frames;                        /**< Number of frames received */
-	int efps;                          /**< Estimated frame-rate      */
+	double efps;                       /**< Estimated frame-rate      */
 	unsigned n_intra;                  /**< Intra-frames decoded      */
 	unsigned n_picup;                  /**< Picture updates sent      */
 	uint32_t ts_min;
 	uint32_t ts_max;
+
+	/** Statistics */
+	struct {
+		uint64_t disp_frames;      /** Total frames displayed     */
+	} stats;
 };
 
 
@@ -296,13 +306,13 @@ static void video_destructor(void *arg)
 
 	tmr_cancel(&vtx->tmr_rtp);
 	mem_deref(vtx->vsrc);
-	lock_write_get(vtx->lock);
+	lock_write_get(vtx->lock_enc);
 	mem_deref(vtx->frame);
 	mem_deref(vtx->mute_frame);
 	mem_deref(vtx->enc);
 	list_flush(&vtx->filtl);
-	lock_rel(vtx->lock);
-	mem_deref(vtx->lock);
+	lock_rel(vtx->lock_enc);
+	mem_deref(vtx->lock_enc);
 
 	/* receive */
 	tmr_cancel(&vrx->tmr_picup);
@@ -319,23 +329,21 @@ static void video_destructor(void *arg)
 }
 
 
-static int get_fps(const struct video *v)
+static double get_fps(const struct video *v)
 {
 	const char *attr;
 
 	/* RFC4566 */
 	attr = sdp_media_rattr(stream_sdpmedia(v->strm), "framerate");
 	if (attr) {
-		/* NOTE: fractional values are ignored */
-		const double fps = atof(attr);
-		return (int)fps;
+		return atof(attr);
 	}
 	else
 		return v->cfg.fps;
 }
 
 
-static int packet_handler(bool marker, uint32_t ts,
+static int packet_handler(bool marker, uint64_t ts,
 			  const uint8_t *hdr, size_t hdr_len,
 			  const uint8_t *pld, size_t pld_len,
 			  void *arg)
@@ -346,14 +354,12 @@ static int packet_handler(bool marker, uint32_t ts,
 	uint32_t rtp_ts;
 	int err;
 
-	/* NOTE: does not handle timestamp wrap around */
-	if (ts < vtx->ts_min)
-		vtx->ts_min = ts;
-	if (ts > vtx->ts_max)
-		vtx->ts_max = ts;
+	if (!vtx->ts_base)
+		vtx->ts_base = ts;
+	vtx->ts_last = ts;
 
 	/* add random timestamp offset */
-	rtp_ts = vtx->ts_offset + ts;
+	rtp_ts = vtx->ts_offset + (ts & 0xffffffff);
 
 	err = vidqent_alloc(&qent, marker, strm->pt_enc, rtp_ts,
 			    hdr, hdr_len, pld, pld_len);
@@ -374,10 +380,12 @@ static int packet_handler(bool marker, uint32_t ts,
  *
  * @note This function has REAL-TIME properties
  *
- * @param vtx   Video transmit object
- * @param frame Video frame to send
+ * @param vtx        Video transmit object
+ * @param frame      Video frame to send
+ * @param timestamp  Frame timestamp in VIDEO_TIMEBASE units
  */
-static void encode_rtp_send(struct vtx *vtx, struct vidframe *frame)
+static void encode_rtp_send(struct vtx *vtx, struct vidframe *frame,
+			    uint64_t timestamp)
 {
 	struct le *le;
 	int err = 0;
@@ -395,7 +403,7 @@ static void encode_rtp_send(struct vtx *vtx, struct vidframe *frame)
 		return;
 	}
 
-	lock_write_get(vtx->lock);
+	lock_write_get(vtx->lock_enc);
 
 	/* Convert image */
 	if (frame->fmt != (enum vidfmt)vtx->video->cfg.enc_fmt) {
@@ -408,7 +416,7 @@ static void encode_rtp_send(struct vtx *vtx, struct vidframe *frame)
 					     vtx->video->cfg.enc_fmt,
 					     &vtx->vsrc_size);
 			if (err)
-				goto unlock;
+				goto out;
 		}
 
 		vidconv(vtx->frame, frame, 0);
@@ -424,35 +432,42 @@ static void encode_rtp_send(struct vtx *vtx, struct vidframe *frame)
 			err |= st->vf->ench(st, frame);
 	}
 
- unlock:
-	lock_rel(vtx->lock);
-
 	if (err)
-		return;
+		goto out;
 
 	/* Encode the whole picture frame */
-	err = vtx->vc->ench(vtx->enc, vtx->picup, frame);
+	err = vtx->vc->ench(vtx->enc, vtx->picup, frame, timestamp);
 	if (err)
-		return;
+		goto out;
 
 	vtx->picup = false;
+
+ out:
+	lock_rel(vtx->lock_enc);
 }
 
 
 /**
  * Read frames from video source
  *
- * @param frame Video frame
- * @param arg   Handler argument
+ * @param frame      Video frame
+ * @param timestamp  Frame timestamp in VIDEO_TIMEBASE units
+ * @param arg        Handler argument
  *
  * @note This function has REAL-TIME properties
  */
-static void vidsrc_frame_handler(struct vidframe *frame, void *arg)
+static void vidsrc_frame_handler(struct vidframe *frame, uint64_t timestamp,
+				 void *arg)
 {
 	struct vtx *vtx = arg;
 
+	/* XXX: save timestamp(s) and pass to encoder */
+	(void)timestamp;
+
 	++vtx->frames;
 
+	++vtx->stats.src_frames;
+
 	/* Is the video muted? If so insert video mute image */
 	if (vtx->muted)
 		frame = vtx->mute_frame;
@@ -461,7 +476,7 @@ static void vidsrc_frame_handler(struct vidframe *frame, void *arg)
 		return;
 
 	/* Encode and send */
-	encode_rtp_send(vtx, frame);
+	encode_rtp_send(vtx, frame, timestamp);
 	vtx->muted_frames++;
 }
 
@@ -480,7 +495,7 @@ static int vtx_alloc(struct vtx *vtx, struct video *video)
 {
 	int err;
 
-	err = lock_alloc(&vtx->lock);
+	err  = lock_alloc(&vtx->lock_enc);
 	err |= lock_alloc(&vtx->lock_tx);
 	if (err)
 		return err;
@@ -496,8 +511,6 @@ static int vtx_alloc(struct vtx *vtx, struct video *video)
 
 	tmr_start(&vtx->tmr_rtp, 1, rtp_tmr_handler, vtx);
 
-	vtx->ts_min = ~0;
-
 	return err;
 }
 
@@ -636,6 +649,8 @@ static int video_stream_decode(struct vrx *vrx, const struct rtp_header *hdr,
 			err |= st->vf->dech(st, frame);
 	}
 
+	++vrx->stats.disp_frames;
+
 	err = vidisp_display(vrx->vidisp, v->peer, frame);
 	frame_filt = mem_deref(frame_filt);
 	if (err == ENODEV) {
@@ -819,12 +834,15 @@ int video_alloc(struct video **vp, const struct stream_param *stream_prm,
 	if (err)
 		goto out;
 
+	if (vidisp_find(baresip_vidispl(), NULL) == NULL)
+		sdp_media_set_ldir(v->strm->sdp, SDP_SENDONLY);
+
 	if (cfg->avt.rtp_bw.max >= AUDIO_BANDWIDTH) {
 		stream_set_bw(v->strm, cfg->avt.rtp_bw.max - AUDIO_BANDWIDTH);
 	}
 
 	err |= sdp_media_set_lattr(stream_sdpmedia(v->strm), true,
-				   "framerate", "%d", v->cfg.fps);
+				   "framerate", "%.2f", v->cfg.fps);
 
 	/* RFC 4585 */
 	err |= sdp_media_set_lattr(stream_sdpmedia(v->strm), true,
@@ -954,8 +972,8 @@ static void tmr_handler(void *arg)
 	tmr_start(&v->tmr, TMR_INTERVAL * 1000, tmr_handler, v);
 
 	/* Estimate framerates */
-	v->vtx.efps = v->vtx.frames / TMR_INTERVAL;
-	v->vrx.efps = v->vrx.frames / TMR_INTERVAL;
+	v->vtx.efps = (double)v->vtx.frames / (double)TMR_INTERVAL;
+	v->vrx.efps = (double)v->vrx.frames / (double)TMR_INTERVAL;
 
 	v->vtx.frames = 0;
 	v->vrx.frames = 0;
@@ -1137,6 +1155,8 @@ int video_encoder_set(struct video *v, struct vidcodec *vc,
 		return ENOENT;
 	}
 
+	lock_write_get(vtx->lock_enc);
+
 	if (vc != vtx->vc) {
 
 		struct videnc_param prm;
@@ -1146,7 +1166,7 @@ int video_encoder_set(struct video *v, struct vidcodec *vc,
 		prm.fps     = get_fps(v);
 		prm.max_fs  = -1;
 
-		info("Set video encoder: %s %s (%u bit/s, %u fps)\n",
+		info("Set video encoder: %s %s (%u bit/s, %.2f fps)\n",
 		     vc->name, vc->variant, prm.bitrate, prm.fps);
 
 		vtx->enc = mem_deref(vtx->enc);
@@ -1154,7 +1174,7 @@ int video_encoder_set(struct video *v, struct vidcodec *vc,
 				  packet_handler, vtx);
 		if (err) {
 			warning("video: encoder alloc: %m\n", err);
-			return err;
+			goto out;
 		}
 
 		vtx->vc = vc;
@@ -1162,6 +1182,9 @@ int video_encoder_set(struct video *v, struct vidcodec *vc,
 
 	stream_update_encoder(v->strm, pt_tx);
 
+ out:
+	lock_rel(vtx->lock_enc);
+
 	return err;
 }
 
@@ -1308,13 +1331,22 @@ static int vtx_debug(struct re_printf *pf, const struct vtx *vtx)
 	err |= re_hprintf(pf, " tx: encode: %s %s\n",
 			  vtx->vc ? vtx->vc->name : "none",
 			  vtx->frame ? vidfmt_name(vtx->frame->fmt) : "?");
-	err |= re_hprintf(pf, "     source: %s %u x %u, fps=%d\n",
+	err |= re_hprintf(pf, "     source: %s %u x %u, fps=%.2f"
+			  " frames=%llu\n",
 			  vtx->vsrc ? vidsrc_get(vtx->vsrc)->name : "none",
 			  vtx->vsrc_size.w,
-			  vtx->vsrc_size.h, vtx->vsrc_prm.fps);
-	err |= re_hprintf(pf, "     skipc=%u\n", vtx->skipc);
-	err |= re_hprintf(pf, "     time = %.3f sec\n",
-			  video_calc_seconds(vtx->ts_max - vtx->ts_min));
+			  vtx->vsrc_size.h, vtx->vsrc_prm.fps,
+			  vtx->stats.src_frames);
+	err |= re_hprintf(pf, "     skipc=%u sendq=%u\n",
+			  vtx->skipc, list_count(&vtx->sendq));
+
+	if (vtx->ts_base) {
+		err |= re_hprintf(pf, "     time = %.3f sec\n",
+			  video_calc_seconds(vtx->ts_last - vtx->ts_base));
+	}
+	else {
+		err |= re_hprintf(pf, "     time = (not started)\n");
+	}
 
 	return err;
 }
@@ -1327,9 +1359,10 @@ static int vrx_debug(struct re_printf *pf, const struct vrx *vrx)
 	err |= re_hprintf(pf, " rx: decode: %s %s\n",
 			  vrx->vc ? vrx->vc->name : "none",
 			  vidfmt_name(vrx->fmt));
-	err |= re_hprintf(pf, "     vidisp: %s %u x %u\n",
+	err |= re_hprintf(pf, "     vidisp: %s %u x %u frames=%llu\n",
 			  vrx->vidisp ? vidisp_get(vrx->vidisp)->name : "none",
-			  vrx->size.w, vrx->size.h);
+			  vrx->size.w, vrx->size.h,
+			  vrx->stats.disp_frames);
 	err |= re_hprintf(pf, "     n_intra=%u, n_picup=%u\n",
 			  vrx->n_intra, vrx->n_picup);
 	err |= re_hprintf(pf, "     time = %.3f sec\n",
@@ -1375,7 +1408,7 @@ int video_print(struct re_printf *pf, const struct video *v)
 	if (!v)
 		return 0;
 
-	return re_hprintf(pf, " efps=%d/%d", v->vtx.efps, v->vrx.efps);
+	return re_hprintf(pf, " efps=%.1f/%.1f", v->vtx.efps, v->vrx.efps);
 }
 
 
